from codec import dump, dumps, load, loads
from geometry import Point, LineString, Polygon
from geometry import MultiLineString, MultiPoint, MultiPolygon
from geometry import GeometryCollection
from feature import Feature, FeatureCollection
from base import GeoJSON





from geojson.base import GeoJSON


class CoordinateReferenceSystem(GeoJSON):

    def __init__(self, properties=None, **extra):
        super(CoordinateReferenceSystem, self).__init__(**extra)
        self.properties = properties or {}

    @property
    def __geo_interface__(self):
        d = super(CoordinateReferenceSystem, self).__geo_interface__
        d.update(type=self.type, properties=self.properties)
        return d


class Named(CoordinateReferenceSystem):

    def __init__(self, properties=None, **extra):
        super(Named, self).__init__(properties=properties, **extra)
        self.type = "name"


class Linked(CoordinateReferenceSystem):
    
    def __init__(self, properties=None, **extra):
        super(Linked, self).__init__(properties=properties, **extra)
        self.type = "link"


class Default(object):
    """GeoJSON default, long/lat WGS84, is not serialized."""
    pass





from geojson.mapping import is_mapping, to_mapping
import geojson
import geojson.factory


class GeoJSON(object):

    def __init__(self, **extra):
        super(GeoJSON, self).__init__()
        self.type = type(self).__name__
        self.extra = extra

    def __repr__(self):
        params = ", ".join("%s=%s" % (key, value) 
                           for (key, value) in self
                           if key != "type")
        return "%s(%s)" % (self.type, params)                           

    def __str__(self):
        return str(self.__geo_interface__)

    def __iter__(self):        
        def iter_geo_interface(ob):
            if not hasattr(ob, "__geo_interface__"):
                return
            for (key, value) in ob.__geo_interface__.items():
                if hasattr(value, "__geo_interface__"):
                    key, value = iter_geo_interface(value)
                yield (key, value)
        return iter_geo_interface(self)

    @property
    def __geo_interface__(self):
        d = dict(**self.extra)
        if self.type != "GeoJSON":
            d.update(type=self.type)
        return d

    @classmethod
    def to_instance(cls, ob, default=None, strict=False):
        """Encode a GeoJSON dict into an GeoJSON object.
        
        Assumes the caller knows that the dict should satisfy a GeoJSON type.
        """
        if ob is None and default is not None:
            instance = default()
        elif isinstance(ob, GeoJSON):
            instance = ob
        else:
            mapping = to_mapping(ob)
            d = dict((str(k), mapping[k]) for k in mapping)
            try:
                type_ = d.pop("type")
                geojson_factory = getattr(geojson.factory, type_)
                if not issubclass(geojson_factory, GeoJSON):
                    raise TypeError("""\
                    Not a valid GeoJSON type:
                    %r (geojson_factory: %r, cls: %r)
                    """ % (type_, geojson_factory, cls))
                instance = geojson_factory(**d)
            except (AttributeError, KeyError), invalid:
                if not strict:
                    instance = ob
                else:
                    msg = "Cannot coerce %r into a valid GeoJSON structure: %s"
                    msg %= (ob, invalid)
                    raise ValueError(msg)
        return instance





# ============================================================================
# GeoJSON. Copyright (C) 2007 Sean C. Gillies
#
# See ../LICENSE.txt
# 
# Contact: Sean Gillies, sgillies@frii.com
# ============================================================================
import unittest
import glob
import os

import doctest

optionflags = (doctest.REPORT_ONLY_FIRST_FAILURE |
               doctest.NORMALIZE_WHITESPACE |
               doctest.ELLIPSIS)

def list_doctests():
    return [filename
            for filename
            in glob.glob(os.path.join(os.path.dirname(__file__), '*.txt'))]

def open_file(filename, mode='r'):
    """Helper function to open files from within the tests package."""
    return open(os.path.join(os.path.dirname(__file__), filename), mode)

def setUp(test):
    test.globs.update(dict(
            open_file = open_file,
            ))

def test_suite():
    return unittest.TestSuite(
        [doctest.DocFileSuite(os.path.basename(filename),
                              optionflags=optionflags,
                              setUp=setUp)
         for filename
         in list_doctests()])



from geojson.geometry import Point, LineString, Polygon
from geojson.geometry import MultiLineString, MultiPoint, MultiPolygon
from geojson.geometry import GeometryCollection
from geojson.feature import Feature, FeatureCollection
from geojson.base import GeoJSON
from geojson.crs import Named, Linked

name = Named
link = Linked



from geojson.base import GeoJSON
import geojson.crs 


class Geometry(GeoJSON):

    """A (WGS84) GIS geometry."""

    def __init__(self, coordinates=None, crs=None, **extra):
        super(Geometry, self).__init__(**extra)
        self.coordinates = coordinates or []
        self.crs = self.to_instance(crs, default=geojson.crs.Default, strict=True)
                    
    @property
    def __geo_interface__(self):
        d = super(Geometry, self).__geo_interface__
        crs = getattr(self.crs, '__geo_interface__', None)
        d.update(coordinates=self.coordinates) 
        if crs is not None:
            d.update(crs=crs)
        return d


class GeometryCollection(GeoJSON):

    """A collection of (WGS84) GIS geometries."""

    def __init__(self, geometries=None, **extra):
        super(GeometryCollection, self).__init__(**extra)
        self.geometries = geometries or []
            
    @property
    def __geo_interface__(self):
        d = super(GeometryCollection, self).__geo_interface__
        geometries = list(self.to_instance(geom, strict=True)
                          for geom in self.geometries)
        d.update(geometries=geometries)
        return d


# Marker classes.

class Point(Geometry): pass


class MultiPoint(Geometry): pass


class LineString(MultiPoint):  pass
    
    
class MultiLineString(Geometry): pass 


class Polygon(Geometry): pass


class MultiPolygon(Geometry): pass




# ============================================================================
# GeoJSON. Copyright (C) 2007 Sean C. Gillies
#
# See ../LICENSE.txt
# 
# Contact: Sean Gillies, sgillies@frii.com
# ============================================================================

import simplejson
import geojson
import geojson.factory
from geojson.mapping import Mapping, to_mapping


class PyGFPEncoder(simplejson.JSONEncoder):

    def default(self, obj):
        if isinstance(obj, Mapping):
            mapping = obj
        else:
            mapping = to_mapping(obj)
        d = dict(mapping)
        type_str = d.pop("type", None)
        if type_str:
            geojson_factory = getattr(geojson.factory, type_str, geojson.factory.GeoJSON)
            d = geojson_factory(**d).__geo_interface__
        return d


# Wrap the functions from simplejson, providing encoder, decoders, and
# object creation hooks

def dump(obj, fp, cls=PyGFPEncoder, **kwargs):
    return simplejson.dump(to_mapping(obj), fp, cls=cls, **kwargs)


def dumps(obj, cls=PyGFPEncoder, **kwargs):
    return simplejson.dumps(to_mapping(obj), cls=cls, **kwargs)


def load(fp, cls=simplejson.JSONDecoder, object_hook=None, **kwargs):
    return simplejson.load(fp, cls=cls, object_hook=object_hook, **kwargs)


def loads(s, cls=simplejson.JSONDecoder, object_hook=None, **kwargs):
    return simplejson.loads(s, cls=cls, object_hook=object_hook, **kwargs)




# ============================================================================
# GeoJSON. Copyright (C) 2007 Sean C. Gillies
#
# See ../LICENSE.txt
# 
# Contact: Sean Gillies, sgillies@frii.com
# ============================================================================

"""
SimpleWebFeature is a working example of a class that satisfies the Python geo
interface.
"""

from geojson.base import GeoJSON
import geojson.geometry 

class Feature(GeoJSON):

    """A (WGS84) GIS Feature."""

    def __init__(self, id=None, geometry=None, properties=None, **extra):
        super(Feature, self).__init__(**extra)
        self.id = id
        self.geometry = self.to_instance(geometry, strict=True)
        self.properties = properties or {}

    @property
    def __geo_interface__(self):
        d = super(Feature, self).__geo_interface__
        d.update(id=self.id,
                 geometry=self.geometry.__geo_interface__,
                 properties=self.properties)
        return d


class FeatureCollection(GeoJSON):

    """A collection of Features."""

    def __init__(self, features, **extra):
        super(FeatureCollection, self).__init__(**extra)
        self.features = features

    @property
    def __geo_interface__(self):
        d = super(FeatureCollection, self).__geo_interface__
        d.update(features=self.features)
        return d
    
   




from setuptools import setup

readme_text = open("README.txt", "rb").read()
version = open("VERSION.txt", "rb").read().strip()

setup(name          = "geojson",
      version       = version,
      description   = "Encoder/decoder for simple GIS features",
      license       = "BSD",
      keywords      = "gis geography json",
      author        = "Sean Gillies",
      author_email  = "sgillies@frii.com",
      maintainer        = "Sean Gillies",
      maintainer_email  = "sgillies@frii.com",
      url           = "http://trac.gispython.org/projects/PCL/wiki/GeoJSON",
      long_description = readme_text,
      packages          = ["geojson"],
      install_requires  = ["simplejson", "setuptools"],
      test_suite    = "tests.test_suite",
      classifiers   = [
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Scientific/Engineering :: GIS",
        ],
)




GEO_INTERFACE_MARKER = "__geo_interface__"


def is_mapping(ob):
    return hasattr(ob, "__getitem__")


def to_mapping(ob):
    if hasattr(ob, GEO_INTERFACE_MARKER):
        candidate = ob.__geo_interface__
        candidate = to_mapping(candidate)
    else:
        candidate = ob
    if not is_mapping(candidate):
        msg = "Expecting something that has %r or a mapping, got %r" 
        msg %= (GEO_INTERFACE_MARKER, ob)
        raise ValueError(msg) 
    return Mapping(candidate)


class Mapping(object):
    
    """Define what a ``mapping`` is to geojson.

    Until py3k, where we have abstract base classes, this will have to do.
    """

    def __init__(self, ob):
        super(Mapping, self).__init__()
        self._ob = ob

    def __contains__(self, name):
        return bool(self.get(name))

    def __getitem__(self, key):
        return self._ob[key]

    def __iter__(self):
        return iter(self._ob)

    def __len__(self):
        return len(self._ob)

    def keys(self):
        return list(self._ob)

    def get(self, key, default=None):
        try:
            value = self._ob[key]
        except KeyError:
            value = default
        return value

    def update(self, other):
        for key in other:
            if not key in self:
                self[key] = other[key]



